/****************************************************************************
*
*    Copyright (c) 2020 Vivante Corporation
*
*    Permission is hereby granted, free of charge, to any person obtaining a
*    copy of this software and associated documentation files (the "Software"),
*    to deal in the Software without restriction, including without limitation
*    the rights to use, copy, modify, merge, publish, distribute, sublicense,
*    and/or sell copies of the Software, and to permit persons to whom the
*    Software is furnished to do so, subject to the following conditions:
*
*    The above copyright notice and this permission notice shall be included in
*    all copies or substantial portions of the Software.
*
*    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
*    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
*    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
*    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
*    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
*    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
*    DEALINGS IN THE SOFTWARE.
*
*****************************************************************************/
#include "vsi_nn_platform.h"
#include "vsi_nn_types.h"
#include "vsi_nn_graph.h"
#include "vsi_nn_log.h"

vsi_status vsi_nn_softmax_compute
    (
    vsi_nn_node_t * self,
    vsi_nn_tensor_t ** inputs,
    vsi_nn_tensor_t ** outputs
    )
{
    vsi_status status;
    vsi_nn_softmax_internal_lcl_data * iter;
    size_t size = sizeof(vx_nn_softmax_params_t);
#ifdef VX_SOFTMAX_AXIS_PARAMETER_SUPPORT
    vx_nn_softmax_params_ext_t paramExt;
    vx_nn_softmax_params_t *param = (vx_nn_softmax_params_t *)&paramExt;

    memset(&paramExt, 0, sizeof(vx_nn_softmax_params_ext_t));
    paramExt.base.beta = self->nn_param.softmax_internal.beta;
    paramExt.axis = 0;

    size = sizeof(vx_nn_softmax_params_ext_t);
#else
    vx_nn_softmax_params_t base;
    vx_nn_softmax_params_t *param = &base;

    memset(&base, 0, sizeof(vx_nn_softmax_params_t));
    base.beta = self->nn_param.softmax_internal.beta;
#endif

    status = VSI_FAILURE;


    status = VSI_FAILURE;
    if(param->beta == 0.f)
    {
        VSILOGW("Softmax's beta is 0. Set beta to 1");
        /* FIXME: Compatible with old case generated by Acuity */
        /* FIXME: for NNAPI case with beta as 0, need refine logical */
        param->beta = 1.f;
    }
    iter = self->nn_param.softmax_internal.data;
    self->n = NULL;
    if(NULL != iter)
    {
        while (iter)
        {
            iter->node = vxSoftmaxLayer2(self->graph->g,
                iter->src_tensor,
                param,
                size,
                iter->dst_tensor);
            if(iter->node == NULL)
            {
                VSILOGE( "Create vxSoftmaxLayer fail." );
                status = VSI_FAILURE;
                break;
            }
            iter = (vsi_nn_softmax_internal_lcl_data *)vsi_nn_LinkListNext((vsi_nn_link_list_t *)iter);
            status = VSI_SUCCESS;
        }
    }
    else
    {
#ifdef VX_SOFTMAX_AXIS_PARAMETER_SUPPORT
        if ( inputs[0]->attr.dim_num > 2 )
        {
            paramExt.axis = 2;
        }
#endif

        self->n = vxSoftmaxLayer2( self->graph->g,
            inputs[0]->t,
            param,
            size,
            outputs[0]->t);

        if( NULL != self->n )
        {
            status = VSI_SUCCESS;
        }
    }

    return status;
} /* vsi_nn_softmax_compute() */
